<?php
//------------------------------------------------------------------------------
	
	include_once('ibbcode.php');
	
//------------------------------------------------------------------------------
	
	class BBCGenericElement extends IBBCElement {
		protected $attributes = array();
		protected $values = array();
		protected $block = false;
		protected $render = false;
		
		public function __construct($parent, $name, $args) {
			parent::__construct($parent, $name, $args);
			
			$this->values = $this->args();
		}
		
		public function block($block = null) {
			if (is_bool($block)) {
				$this->block = $block;
				return $this;
			}
			
			return $this->block;
		}
		
		public function render($render = null) {
			if (is_bool($render)) {
				$this->render = $render;
				return $this;
			}
			
			return $this->render;
		}
		
		protected function content($name) {
			if ($this->children()) {
				$this->attributes["$name"] = htmlentities(parent::__toString());
			}
		}
		
		protected function shift($name) {
			if ($this->values) {
				$this->attributes["$name"] = htmlentities(array_shift($this->values));
			}
		}
		
		protected function remainder($name) {
			if ($this->values) {
				$this->attributes["$name"] = htmlentities(implode($this->values, " "));
			}
		}
		
		public function __toString() {
			$name = $this->name();
			$attributes = "";
			
			$translate = array(
				"i"	=> "em",
				"b"	=> "strong"
			);
			
			$name = (isset($translate["{$name}"]) ? $translate["{$name}"] : $name);
			
			$this->remainder("title");
			
			foreach ($this->attributes as $key => $value) {
				$attributes .= ' ' . $key . '="' . $value . '"';
			}
			
			if ($this->render()) {
				$content = BBCRender($this) . "\n";
			} else {
				$content = parent::__toString();
			}
			
			if ($this->block()) {
				return "\n<{$name}{$attributes}>{$content}</{$name}>";
			}
			
			return "<{$name}{$attributes}>{$content}</{$name}>";
		}
	}
	
	function BBCRender($parent) {
		$default = array(
			'type'	=> 'inline',
			'value'	=> ''
		);
		$segments = array($default);
		$output = '';
		
		// Split into segments:
		foreach ($parent->children() as $child) {
			// New segment:
			if (method_exists($child, 'block') and $child->block()) {
				array_push($segments, 
					array(
						'type'	=> 'block',
						'value'	=> "$child"
					)
				);
				
				array_push($segments, $default);
				
			// Inline text:
			} else {
				$current = array_pop($segments);
				
				$current['value'] .= "$child";
				
				array_push($segments, $current);
			}
		}
		
		// Join together again:
		foreach($segments as $current) {
			if ($current['type'] == 'block') {
				$output .= $current['value'];
				
			} else if (trim($current['value'])) {
				$value = preg_replace('/((\r\n|\n)\s*){2,}/', "</p>\n<p>", trim($current['value']));
				$output .= "\n<p>$value</p>";
			}
		}
		
		return $output;
	}
	
//------------------------------------------------------------------------------
?>
